
package org.cocos2dx.cpp;


import org.cocos2dx.cpp.BaseService;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * @说 明: 蓝牙通讯服务
 * @作 者: jinhui
 * @日 期: 2015/4/25
 */
public class BluetoothService extends BaseService {

    private static final String TAG = BluetoothService.class.getSimpleName();
    private BluetoothManager bluetoothManager;
    private BluetoothAdapter bluetoothAdapter;
    private BluetoothGatt bluetoothGatt;
    private String bluetoothDeviceAddress;
    
    public static final UUID CCCD = UUID.fromString("00002902-0000-1000-8000-00805f9b34fb");
    public static final UUID RX_SERVICE_UUID = UUID.fromString("6e400001-b5a3-f393-e0a9-e50e24dcca9e");
    public static final UUID RX_CHAR_UUID = UUID.fromString("6e400002-b5a3-f393-e0a9-e50e24dcca9e");
    public static final UUID TX_CHAR_UUID = UUID.fromString("6e400003-b5a3-f393-e0a9-e50e24dcca9e");
    
    public BluetoothGatt GetBluetoothGatt()
    {
    	return bluetoothGatt;
    }
    
    /**
     * 连接状态
     */
    private int connectionState = STATE_DISCONNECTED;

    /**
     * 获取连接状态
     *
     * @return STATE_DISCONNECTED:断开状态 <p>
     * STATE_CONNECTING:连通状态 <p>
     * STATE_CONNECTED :保持通话状态 <p>
     */
    public int getConnectionState() {
        return connectionState;
    }

    /**
     * 断开状态
     */
    public static final int STATE_DISCONNECTED = 0;
    /**
     * 连通状态
     */
    public static final int STATE_CONNECTING = 1;
    /**
     * 保持通话状态
     */
    public static final int STATE_CONNECTED = 2;
    /**
     * 连接到服务器GATT
     */
    public final static String ACTION_GATT_CONNECTED = "ACTION_GATT_CONNECTED";
    /**
     * 从GATT服务器断开连接
     */
    public final static String ACTION_GATT_DISCONNECTED = "ACTION_GATT_DISCONNECTED";
    /**
     * 发现GATT服务
     */
    public final static String ACTION_GATT_SERVICES_DISCOVERED = "ACTION_GATT_SERVICES_DISCOVERED";
    /**
     * 从设备接收到的数据，这可以是一个读或通知
     */
    public final static String ACTION_DATA_AVAILABLE = "ACTION_DATA_AVAILABLE";
    /**
     * 额外数据
     */
    public final static String EXTRA_DATA = "EXTRA_DATA";

    /**
     * 回调接口
     */
    private final BluetoothGattCallback gattCallback =
    new BluetoothGattCallback() {
        @Override
        //连接状态变化
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            String intentAction;
            if (newState == BluetoothProfile.STATE_CONNECTED) {
                intentAction = ACTION_GATT_CONNECTED;
                connectionState = STATE_CONNECTED;
                broadcastUpdate(intentAction);
                Log.i(TAG, "Connected to GATT server.");
                Log.i(TAG, "Attempting to start service discovery:" +
                        bluetoothGatt.discoverServices());
            } else if (newState == BluetoothProfile.STATE_DISCONNECTED) {
                intentAction = ACTION_GATT_DISCONNECTED;
                connectionState = STATE_DISCONNECTED;
                Log.i(TAG, "Disconnected from GATT server.");
                broadcastUpdate(intentAction);
            }
        }

        @Override
        //发现新服务
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                broadcastUpdate(ACTION_GATT_SERVICES_DISCOVERED);
            } else {
                Log.w(TAG, "onServicesDiscovered received: " + status);
            }
        }

        @Override
        // 特征的读作操作的结果
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
            	Log.d(TAG, "status == BluetoothGatt.GATT_SUCCESS");
                broadcastUpdate(ACTION_DATA_AVAILABLE, characteristic);
            }
            else{
            	Log.e(TAG, "status != BluetoothGatt.GATT_SUCCESS");
            }
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            broadcastUpdate(ACTION_DATA_AVAILABLE, characteristic);
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            super.onCharacteristicWrite(gatt, characteristic, status);
            if (status == BluetoothGatt.GATT_SUCCESS) {
            	//Log.d(TAG,"已经成功写入数据");
            	//Log.d(TAG,"send:"+characteristic.getValue());
            }
        }
    };

    /**
     * 发送状态变化广播
     *
     * @param action
     */
    private void broadcastUpdate(final String action) {
        final Intent intent = new Intent(action);
        sendBroadcast(intent);
    }

    private void broadcastUpdate(String action, BluetoothGattCharacteristic characteristic) {
    	Log.e(TAG, "broadcastUpdate");
        Intent intent = new Intent(action);
        byte[] data = characteristic.getValue();
        if (data != null && data.length > 0) {
            for (int i = 0; i < data.length; i++) {
                data[i] &= 0xFF;
            }
            intent.putExtra(EXTRA_DATA, data);
        }
        sendBroadcast(intent);
    }
    
    @Override
    public boolean onUnbind(Intent intent) {
        // 使用给定的设备后,你应该确保BluetoothGatt.close(),
        // 在这个特定的例子中,close()调用UI时断开服务。
        close();
        return super.onUnbind(intent);
    }

    /**
     * 初始化本地蓝牙适配器的引用
     *
     * @return true:初始化成功
     */
    public boolean initialize() {
        // API级别18以上,通过BluetoothManager BluetoothAdapter引用
        if (bluetoothManager == null) {
            bluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
            if (bluetoothManager == null) {
                Log.e(TAG, "无法初始化BluetoothManager");
                return false;
            }
        }
        bluetoothAdapter = bluetoothManager.getAdapter();
        if (bluetoothAdapter == null) {
            Log.e(TAG, "无法获得BluetoothAdapter");
            return false;
        }
        return true;
    }

    /**
     * 连接到蓝牙设备服务
     *
     * @param address 目标设备的设备地址
     * @return true：连接成功了。通过异步连接结果报告
     * {@code BluetoothGattCallback#onConnectionStateChange(android.bluetooth.BluetoothGatt, int, int)}
     * callback.
     */
    public boolean connect(final String address) {
        if (bluetoothAdapter == null) {
            Log.e(TAG, "BluetoothAdapter没有初始化");
            initialize();
        }
        if (bluetoothAdapter == null) {
            return false;
        }
        if (address == null) {
            Log.e(TAG, "BluetoothAdapter未指明的地址");
            return false;
        }
        
        // 尝试重新连接以前连接设备
        /*if (bluetoothDeviceAddress != null && address.equals(bluetoothDeviceAddress)
                && bluetoothGatt != null) {
            Log.d(TAG, "尝试使用一个现有bluetoothGatt连接");
            if (bluetoothGatt.connect()) {
                connectionState = STATE_CONNECTING;
                return true;
            } 
            else {
            	Log.e(TAG, "尝试使用一个现有bluetoothGatt连接 连接失败");
                return false;
            }
        }*/
        
        final BluetoothDevice device = bluetoothAdapter.getRemoteDevice(address);
        if (device == null) {
            Log.e(TAG, "没有找到该设备，无法连接");
            return false;
        }
        
        if(bluetoothGatt != null)
        {
            bluetoothGatt.close();
            bluetoothGatt = null;
        }
        
        // 想直接连接到设备,所以参数设置为false
        bluetoothGatt = device.connectGatt(this, false, gattCallback);
        Log.d(TAG, "试图创建一个新的连接");
        bluetoothDeviceAddress = address;
        connectionState = STATE_CONNECTING;
        return true;
    }

    /**
     * 取消或断开一个现有的连接，通过异步分离结果报告
     * {@code BluetoothGattCallback#onConnectionStateChange(android.bluetooth.BluetoothGatt, int, int)}
     * callback.
     */
    public void disconnect() {
        if (bluetoothAdapter == null || bluetoothGatt == null) {
            Log.w(TAG, "BluetoothAdapter没有初始化");
            return;
        }
        bluetoothGatt.disconnect();
    }

    /**
     * 使用给定的BLE设备后,应用程序必须调用这个方法,以确保正确地释放资源
     */
    public void close() {
        if (bluetoothGatt == null) {
            return;
        }
        bluetoothGatt.close();
        bluetoothGatt = null;
    }

    /**
     * 读取数据 {@code BluetoothGattCharacteristic}.
     * 通过异步读取的结果报告给 {@code BluetoothGattCallback#onCharacteristicRead(android.bluetooth.BluetoothGatt, android.bluetooth.BluetoothGattCharacteristic, int)}
     * callback.
     *
     * @param characteristic 读取的特点
     */
    public void readCharacteristic(BluetoothGattCharacteristic characteristic) {
        if (bluetoothAdapter == null || bluetoothGatt == null) {
            Log.d(TAG, "BluetoothAdapter没有初始化");
            return;
        }
        boolean b = bluetoothGatt.readCharacteristic(characteristic);
        Log.d(TAG, "******** readCharacteristic: "+b);
    }

    /**
     * 写入数据
     *
     * @param characteristic 写入的特点
     */
    public void writeCharacteristic(BluetoothGattCharacteristic characteristic) {
        if (bluetoothAdapter == null || bluetoothGatt == null) {
            Log.d(TAG, "BluetoothAdapter没有初始化");
            return;
        }
        
        //if(characteristic.getUuid().equals(RX_CHAR_UUID)){
            boolean b = bluetoothGatt.writeCharacteristic(characteristic);
            //if(!b) Log.e(TAG, "写入数据失败");
            //else Log.d(TAG, "写入数据成功");
        //}

    }
    
    /**
     * 写入数据
     *
     * @param characteristic 发送数据
     */
    public void writeCharacteristic(byte[] buf) {
    	BluetoothGattService RxService = bluetoothGatt.getService(RX_SERVICE_UUID);
    	if (RxService == null) {
    		Log.e(TAG, "RxService == null");
            return;
        }
    	BluetoothGattCharacteristic sendCharacteristic = RxService.getCharacteristic(RX_CHAR_UUID);
        if (sendCharacteristic == null) {
        	Log.e(TAG, "sendCharacteristic == null");
            return;
        }   
        
        sendCharacteristic.setValue(buf);
        boolean b = bluetoothGatt.writeCharacteristic(sendCharacteristic);
        if(!b) Log.e(TAG, "写入数据失败");
        else Log.d(TAG, "写入数据成功");
    }
    

    /**
     * 启用或禁用通知给特征
     *
     * @param characteristic 特点行动
     * @param enabled        true：启用通知
     */
    public void setCharacteristicNotification(BluetoothGattCharacteristic characteristic, boolean enabled,int isMiniRobot) {
        if (bluetoothAdapter == null || bluetoothGatt == null) {
            Log.e(TAG, "BluetoothAdapter not initialized");
            return;
        }
        if (bluetoothGatt != null)
            bluetoothGatt.setCharacteristicNotification(characteristic, enabled);
        else Log.e(TAG, "bluetoothGatt == null");
        
        if(isMiniRobot == 0) return;
        
        BluetoothGattDescriptor clientConfig = characteristic.getDescriptor(CCCD);
		if (clientConfig == null)
		{
			Log.e(TAG, "clientConfig == null");
	        return;
		}

		/*
		if (enabled) {
	        clientConfig.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
		} else {
	        clientConfig.setValue(BluetoothGattDescriptor.DISABLE_NOTIFICATION_VALUE);
		}
		*/

		clientConfig.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
		bluetoothGatt.writeDescriptor(clientConfig);
		Log.d(TAG, "clientConfig != null");
    }

    /**
     * 检索设备支持的服务列表，这应该是之后才调用
     * {@code BluetoothGatt#discoverServices()} 成功完成
     *
     * @return 支持的服务列表
     */
    public List<BluetoothGattService> getSupportedGattServices() {
        if (bluetoothGatt == null)
        {
        	Log.e(TAG,"bluetoothGatt == null");
        	return new ArrayList<BluetoothGattService>();
        }
     
        return bluetoothGatt.getServices();
    }
    
    public void enableTXNotification()
    { 
    	Log.d(TAG, "enableTXNotification");
    	BluetoothGattService RxService = bluetoothGatt.getService(RX_SERVICE_UUID);
    	if (RxService == null) {
    		Log.e(TAG, "RxService == null");
            return;
        }
    	BluetoothGattCharacteristic TxChar = RxService.getCharacteristic(TX_CHAR_UUID);
        if (TxChar == null) {
        	Log.e(TAG, "TxChar == null");
            return;
        }
        
        bluetoothGatt.setCharacteristicNotification(TxChar,true);	
        
        BluetoothGattDescriptor descriptor = TxChar.getDescriptor(CCCD);
        descriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
        bluetoothGatt.writeDescriptor(descriptor);
    }
}
